import json

from ..exceptions import SparkPostAPIException as RequestsSparkPostAPIException


class SparkPostAPIException(RequestsSparkPostAPIException):
    def __init__(self, response, *args, **kwargs):
        errors = None
        # noinspection PyBroadException
        try:
            data = json.loads(response.body.decode("utf-8"))
            if data:
                errors = data['errors']
                errors = [e['message'] + ': ' + e.get('description', '')
                          for e in errors]
        # TODO: select exception to catch here
        except:  # noqa: E722
            pass
        if not errors:
            errors = [response.body.decode("utf-8") or ""]
        self.status = response.code
        self.response = response
        self.errors = errors
        message = """Call to {uri} returned {status_code}, errors:

        {errors}
        """.format(
            uri=response.effective_url,
            status_code=response.code,
            errors='\n'.join(errors)
        )
        super(RequestsSparkPostAPIException, self).__init__(message, *args,
                                                            **kwargs)



import json

from .base import Resource


class Templates(Resource):
    """
    Templates class used to create, update, delete, list and get templates. For
    detailed request and response formats, see the `Templates API documentation
    <https://www.sparkpost.com/api#/reference/templates>`_.
    """

    key = 'templates'

    def _translate_keys(self, **kwargs):
        model = {
            'content': {},
            'options': {}
        }

        if 'id' in kwargs:
            model['id'] = kwargs.get('id')
        model['name'] = kwargs.get('name')
        model['description'] = kwargs.get('description')
        model['published'] = kwargs.get('published')

        model['options']['open_tracking'] = kwargs.get('track_opens')
        model['options']['click_tracking'] = kwargs.get('track_clicks')
        model['options']['transactional'] = kwargs.get('is_transactional')

        model['content']['html'] = kwargs.get('html')
        model['content']['text'] = kwargs.get('text')
        model['content']['subject'] = kwargs.get('subject')
        model['content']['from'] = kwargs.get('from_email')
        model['content']['reply_to'] = kwargs.get('reply_to')
        model['content']['headers'] = kwargs.get('custom_headers')

        return model

    def create(self, **kwargs):
        """
        Create a template based on the supplied parameters

        :param str id: ID used to reference the template
        :param str name: Editable display name
        :param str description: Detailed description of the template
        :param bool published: Defaults to False. Whether the template is a
            published or draft version

        :param bool track_opens: Defaults to transmission level setting.
            Used to track opens of transmission
        :param bool track_clicks: Defaults to transmission level setting.
            Used to track clicks of transmission
        :param bool is_transactional: Defaults to transmission level setting.
            Distinguishes between transactional and non-transactional messages
            for unsubscribe and suppression purposes

        :param str html: HTML part of template
        :param str text: Text part of template
        :param str subject: Subject of template
        :param str from_email: Friendly from of template, domain must be a
            verified sending domain to your account or template create
            will fail
        :param str reply_to: Reply to of template
        :param dict custom_headers: Used to set any headers associated with
            template

        :returns: a ``dict`` with the ID
        :raises: :exc:`SparkPostAPIException` if template uses an unverified
            sending domain or there's a syntax error in the content
        """
        payload = self._translate_keys(**kwargs)
        results = self.request('POST', self.uri, data=json.dumps(payload))
        return results

    def update(self, template_id, **kwargs):
        """
        Update a template by ID based on the supplied parameters

        :param str template_id: ID of the template you want to retrieve

        :param str name: Editable display name
        :param str description: Detailed description of the template
        :param bool published: Defaults to False. Whether the template is a
            published or draft version

        :param bool track_opens: Defaults to transmission level setting.
            Used to track opens of transmission
        :param bool track_clicks: Defaults to transmission level setting.
            Used to track clicks of transmission
        :param bool is_transactional: Defaults to transmission level setting.
            Distinguishes between transactional and non-transactional messages
            for unsubscribe and suppression purposes

        :param str html: HTML part of template
        :param str text: Text part of template
        :param str subject: Subject of template
        :param str from_email: Friendly from of template, domain must be a
            verified sending domain to your account or template create
            will fail
        :param str reply_to: Reply to of template
        :param dict custom_headers: Used to set any headers associated with
            template

        :returns: TODO
        :raises: :exc:`SparkPostAPIException` if template is not found
        """
        uri = "%s/%s" % (self.uri, template_id)
        payload = self._translate_keys(**kwargs)
        results = self.request('PUT', uri, data=json.dumps(payload))
        return results

    def delete(self, template_id):
        """
        Delete a template by ID

        :param str template_id: ID of the template you want to delete

        :returns: TODO
        :raises: :exc:`SparkPostAPIException` if template is not found or if
            template is in use
        """
        uri = "%s/%s" % (self.uri, template_id)
        results = self.request('DELETE', uri)
        return results

    def get(self, template_id, draft=None):
        """
        Get a template by ID

        :param str template_id: ID of the template you want to retrieve
        :param bool draft: Defaults to None. If True, returns the most
            recent draft template. If False, returns the most recent published
            template. If None, returns the most recent template version
            regardless of draft or published.

        :returns: the requested template if found
        :raises: :exc:`SparkPostAPIException` if template is not found
        """
        uri = "%s/%s" % (self.uri, template_id)
        params = {}
        if draft is not None:
            params['draft'] = str(draft).lower()
        results = self.request('GET', uri, params=params)
        return results

    def list(self):
        """
        Get a list of your templates

        :returns: list of templates
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        results = self.request('GET', self.uri)
        return results

    def preview(self, template_id, substitution_data, draft=None):
        """
        Get a preivew of your template by ID with the
        provided substitution_data

        :param str template_id: ID of the template you want to retrieve
        :param dict substitution_data: data to be substituted in the
            template content
        :param bool draft: Defaults to None. If True, previews the most
            recent draft template. If False, previews the most recent published
            template. If None, previews the most recent template version
            regardless of draft or published.

        :returns: the requested template if found with content expanded using
            substitution data provided
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        uri = "%s/%s/preview" % (self.uri, template_id)
        params = {}
        if draft is not None:
            params['draft'] = str(draft).lower()
        data = json.dumps({'substitution_data': substitution_data})
        results = self.request('POST',
                               uri,
                               params=params,
                               data=data)
        return results



from django.conf import settings
from django.core.mail.backends.base import BaseEmailBackend

from sparkpost import SparkPost

from .message import SparkPostMessage


class SparkPostEmailBackend(BaseEmailBackend):
    """
    SparkPost wrapper for Django email backend
    """

    def __init__(self, fail_silently=False, **kwargs):
        super(SparkPostEmailBackend, self)\
            .__init__(fail_silently=fail_silently, **kwargs)

        sp_api_key = getattr(settings, 'SPARKPOST_API_KEY', None)

        self.client = SparkPost(sp_api_key)

    def send_messages(self, email_messages):
        """
        Send emails, returns integer representing number of successful emails
        """
        success = 0
        for message in email_messages:
            try:
                response = self._send(SparkPostMessage(message))
                success += response['total_accepted_recipients']
            except Exception:
                if not self.fail_silently:
                    raise
        return success

    def _send(self, message):
        params = getattr(settings, 'SPARKPOST_OPTIONS', {}).copy()
        params.update(message)
        return self.client.transmissions.send(**params)



from .utils import wrap_future
from ..transmissions import Transmissions as SyncTransmissions


class Transmissions(SyncTransmissions):
    def get(self, transmission_id):
        results = self._fetch_get(transmission_id)
        return wrap_future(results, lambda f: f["transmission"])



from tornado.concurrent import Future


def wrap_future(future, convert):
    wrapper = Future()

    def handle_future(future):
        try:
            wrapper.set_result(convert(future.result()))
        except Exception as ex:
            wrapper.set_exception(ex)

    future.add_done_callback(handle_future)
    return wrapper



import pytest
import responses

from sparkpost import SparkPost
from sparkpost.exceptions import SparkPostAPIException


@responses.activate
def test_success_campaigns():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/metrics/campaigns',
        status=200,
        content_type='application/json',
        body='{"results": {"campaigns": []}}'
    )
    sp = SparkPost('fake-key')
    results = sp.metrics.campaigns.list()
    assert results == []


@responses.activate
def test_fail_campaigns():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/metrics/campaigns',
        status=500,
        content_type='application/json',
        body="""
        {"errors": [{"message": "You failed", "description": "More Info"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.metrics.campaigns.list()


@responses.activate
def test_success_domains():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/metrics/domains',
        status=200,
        content_type='application/json',
        body='{"results": {"domains": []}}'
    )
    sp = SparkPost('fake-key')
    results = sp.metrics.domains.list()
    assert results == []


@responses.activate
def test_fail_domains():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/metrics/domains',
        status=500,
        content_type='application/json',
        body="""
        {"errors": [{"message": "You failed", "description": "More Info"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.metrics.domains.list()



import json

from .base import Resource


class RecipientLists(Resource):
    """
    RecipientLists class used to create, update, delete, list and get recipient
    lists. For detailed request and response formats, see the `Recipient Lists
    API documentation
    <https://www.sparkpost.com/api#/reference/recipient-lists>`_.
    """

    key = 'recipient-lists'

    def _translate_keys(self, **kwargs):
        model = {}

        if 'id' in kwargs:
            model['id'] = kwargs.get('id')
        model['name'] = kwargs.get('name')
        model['description'] = kwargs.get('description')
        model['attributes'] = kwargs.get('attributes')
        model['recipients'] = kwargs.get('recipients')

        return model

    def create(self, **kwargs):
        """
        Create a recipient list based on the supplied parameters

        :param str id: ID used to reference the recipient list
        :param str name: Editable display name
        :param str description: Detailed description of the recipient list
        :param dict attributes: Arbitrary metadata related to the list
        :param list recipients: Array of recipient dicts

        :returns: a ``dict`` with the ID, name, and number of accepted
            and rejected recipients
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        payload = self._translate_keys(**kwargs)
        results = self.request('POST', self.uri, data=json.dumps(payload))
        return results

    def update(self, list_id, **kwargs):
        """
        Update a recipient list by ID based on the supplied parameters

        :param str list_id: ID of the recipient list you want to update
        :param str name: Editable display name
        :param str description: Detailed description of the recipient list
        :param dict attributes: Arbitrary metadata related to the list
        :param list recipients: Array of recipient dicts

        :returns: a ``dict`` with the ID, name, and number of accepted
            and rejected recipients
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        uri = "%s/%s" % (self.uri, list_id)
        payload = self._translate_keys(**kwargs)
        results = self.request('PUT', uri, data=json.dumps(payload))
        return results

    def delete(self, list_id):
        """
        Delete a recipient list by ID

        :param str list_id: ID of the recipient list you want to delete

        :returns: empty ``dict``
        :raises: :exc:`SparkPostAPIException` if recipient list is not found
            or if recipient list is in use
        """
        uri = "%s/%s" % (self.uri, list_id)
        results = self.request('DELETE', uri)
        return results

    def get(self, list_id, show_recipients=None):
        """
        Get a recipient list by ID

        :param str list_id: ID of the recipient list you want to retrieve
        :param bool show_recipients: If True, returns attributes for
            all recipients

        :returns: the requested recipient list if found
        :raises: :exc:`SparkPostAPIException` if recipient list is not found
        """
        uri = "%s/%s" % (self.uri, list_id)
        params = {}
        if show_recipients is not None:
            params['show_recipients'] = str(show_recipients).lower()
        results = self.request('GET', uri, params=params)
        return results

    def list(self):
        """
        Get a list of your recipient lists

        :returns: list of recipient lists
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        results = self.request('GET', self.uri)
        return results



"""
This script demonstrates how to use the SparkPostAPIException class. This
particular example will yield output similar to the following:

$ python send_transmission_exception.py
400
{u'errors': [{u'message': u'Invalid domain', u'code': u'7001', u'description':
u'Unconfigured Sending Domain <some-domain-you-havent-configured.com> '}]}
['Invalid domain Code: 7001 Description: Unconfigured Sending Domain
<some-domain-you-havent-configured.com>  \n']
"""

from sparkpost import SparkPost
from sparkpost.exceptions import SparkPostAPIException

sp = SparkPost()

try:
    response = sp.transmissions.send(
        recipients=['john.doe@example.com'],
        text='Hello there',
        from_email='Testing <test@some-domain-you-havent-configured.com>',
        subject='Testing python-sparkpost exceptions'
    )
except SparkPostAPIException as err:
    # http response status code
    print(err.status)
    # python requests library response object
    # http://docs.python-requests.org/en/master/api/#requests.Response
    print(err.response.json())
    # list of formatted errors
    print(err.errors)



from sparkpost import SparkPost

sp = SparkPost()
result = sp.suppression_list.create({
    'email': 'test@test.com',
    'transactional': False,
    'non_transactional': True,
    'description': 'Test description'
})
print(result)



import pytest
import responses

from sparkpost import SparkPost
from sparkpost.exceptions import SparkPostAPIException


@responses.activate
def test_success_list():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/suppression-list',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )

    sp = SparkPost('fake-key')
    results = sp.suppression_list.list()
    assert results == 'yay'


@responses.activate
def test_fail_list():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/suppression-list',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "cant find", "description": "where you go"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.suppression_list.list()


@responses.activate
def test_success_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/suppression-list/foobar',
        status=200,
        content_type='application/json',
        body='{"results": []}'
    )
    sp = SparkPost('fake-key')
    results = sp.suppression_list.get('foobar')
    assert results == []


@responses.activate
def test_fail_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/suppression-list/foobar',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "cant find", "description": "where you go"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.suppression_list.get('foobar')


@responses.activate
def test_success_delete():
    responses.add(
        responses.DELETE,
        'https://api.sparkpost.com/api/v1/suppression-list/foobar',
        status=204,
        content_type='application/json'
    )
    sp = SparkPost('fake-key')
    results = sp.suppression_list.delete('foobar')
    assert results is True


@responses.activate
def test_fail_delete():
    responses.add(
        responses.DELETE,
        'https://api.sparkpost.com/api/v1/suppression-list/foobar',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "cant find", "description": "where you go"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.suppression_list.delete('foobar')


@responses.activate
def test_success_upsert():
    responses.add(
        responses.PUT,
        'https://api.sparkpost.com/api/v1/suppression-list/foobar',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )

    sp = SparkPost('fake-key')
    results = sp.suppression_list._upsert({"email": "foobar"})
    assert results == 'yay'


@responses.activate
def test_success_create():
    responses.add(
        responses.PUT,
        'https://api.sparkpost.com/api/v1/suppression-list/foobar',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )

    sp = SparkPost('fake-key')
    results = sp.suppression_list.create({"email": "foobar"})
    assert results == 'yay'


@responses.activate
def test_success_update():
    responses.add(
        responses.PUT,
        'https://api.sparkpost.com/api/v1/suppression-list/foobar',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )

    sp = SparkPost('fake-key')
    results = sp.suppression_list.update({"email": "foobar"})
    assert results == 'yay'


@responses.activate
def test_success_upsert_bulk():
    responses.add(
        responses.PUT,
        'https://api.sparkpost.com/api/v1/suppression-list',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )

    sp = SparkPost('fake-key')
    results = sp.suppression_list._upsert([
        {"email": "foobar1"}, {"email": "foobar2"}
    ])
    assert results == 'yay'


@responses.activate
def test_fail_upsert():
    responses.add(
        responses.PUT,
        'https://api.sparkpost.com/api/v1/suppression-list/foobar',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "cant find", "description": "where you go"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.suppression_list._upsert({"email": "foobar"})



from sparkpost import SparkPost

sp = SparkPost()
response = sp.recipient_lists.update(
    'EXISTING_TEST_ID',
    name='Test Recipient list',
    recipients=[
        {
            'address': {
                'email': 'test1@test.com'
            }
        },
        {
            'address': {
                'email': 'test2@test.com'
            }
        },
        {
            'address': {
                'email': 'test3@test.com'
            }
        }
    ]
)
print(response)



from sparkpost import SparkPost

sp = SparkPost()
recipients = [
    {
        'email': 'test1@test.com',
        'transactional': False,
        'non_transactional': True,
        'description': 'Test description 1'
    },
    {
        'email': 'test2@test.com',
        'transactional': True,
        'non_transactional': True,
        'description': 'Test description 2'
    },
    {
        'email': 'test3@test.com',
        'transactional': True,
        'non_transactional': False,
        'description': 'Test description 3'
    }
]
result = sp.suppression_list.create(recipients)
print(result)



try:
    from urllib.parse import urlparse
# TODO: select exception to catch here
except:  # noqa: E722
    from urlparse import urlparse

import pytest
import responses

from sparkpost import SparkPost
from sparkpost import Templates
from sparkpost.exceptions import SparkPostAPIException


def test_translate_keys_with_id():
    t = Templates('uri', 'key')
    results = t._translate_keys(id='test_id')
    assert results['id'] == 'test_id'


@responses.activate
def test_success_create():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/templates',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )

    sp = SparkPost('fake-key')
    results = sp.templates.create()
    assert results == 'yay'


@responses.activate
def test_fail_create():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/templates',
        status=500,
        content_type='application/json',
        body="""
        {"errors": [{"message": "You failed", "description": "More Info"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.templates.create()


@responses.activate
def test_success_update():
    responses.add(
        responses.PUT,
        'https://api.sparkpost.com/api/v1/templates/foobar',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )

    sp = SparkPost('fake-key')
    results = sp.templates.update('foobar', name='foobar')
    assert results == 'yay'


@responses.activate
def test_fail_update():
    responses.add(
        responses.PUT,
        'https://api.sparkpost.com/api/v1/templates/foobar',
        status=500,
        content_type='application/json',
        body="""
        {"errors": [{"message": "You failed", "description": "More Info"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.templates.update('foobar', name='foobar')


@responses.activate
def test_success_delete():
    responses.add(
        responses.DELETE,
        'https://api.sparkpost.com/api/v1/templates/foobar',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )

    sp = SparkPost('fake-key')
    results = sp.templates.delete('foobar')
    assert results == 'yay'


@responses.activate
def test_fail_delete():
    responses.add(
        responses.DELETE,
        'https://api.sparkpost.com/api/v1/templates/foobar',
        status=500,
        content_type='application/json',
        body="""
        {"errors": [{"message": "You failed", "description": "More Info"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.templates.delete('foobar')


@responses.activate
def test_success_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/templates/foobar',
        status=200,
        content_type='application/json',
        body='{"results": {}}'
    )
    sp = SparkPost('fake-key')
    results = sp.templates.get('foobar')
    assert results == {}


@responses.activate
def test_success_get_with_is_draft():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/templates/foobar?draft=true',
        match_querystring=True,
        status=200,
        content_type='application/json',
        body='{"results": {}}'
    )
    sp = SparkPost('fake-key')
    results = sp.templates.get('foobar', True)
    assert results == {}


@responses.activate
def test_fail_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/templates/foobar',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "cant find", "description": "where you go"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.templates.get('foobar')


@responses.activate
def test_success_list():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/templates',
        status=200,
        content_type='application/json',
        body='{"results": []}'
    )
    sp = SparkPost('fake-key')
    response = sp.templates.list()
    assert response == []


@responses.activate
def test_success_preview():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/templates/foobar/preview',
        status=200,
        content_type='application/json',
        body='{"results": {}}'
    )
    sp = SparkPost('fake-key')
    results = sp.templates.preview('foobar', {})
    assert responses.calls[0].request.body == '{"substitution_data": {}}'
    assert results == {}


@responses.activate
def test_success_preview_with_draft():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/templates/foobar/preview?draft=true',
        match_querystring=True,
        status=200,
        content_type='application/json',
        body='{"results": {}}'
    )
    sp = SparkPost('fake-key')
    results = sp.templates.preview('foobar', {}, True)
    parsed = urlparse(responses.calls[0].request.url)
    assert parsed.query == 'draft=true'
    assert results == {}


@responses.activate
def test_fail_preview():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/templates/foobar/preview',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "cant find", "description": "where you go"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.templates.preview('foobar', {})



import pytest
import mock

from django.conf import settings
from django.core.mail import send_mail
from django.core.mail import send_mass_mail
from django.core.mail import EmailMultiAlternatives
from django.utils.functional import empty

from sparkpost.django.email_backend import SparkPostEmailBackend
from sparkpost.django.exceptions import UnsupportedContent
from sparkpost.transmissions import Transmissions

API_KEY = 'API_Key'


def reconfigure_settings(**new_settings):
    old_settings = settings._wrapped

    settings._wrapped = empty
    settings.configure(default_settings=old_settings, **new_settings)


reconfigure_settings(
    DEBUG=True,
    EMAIL_BACKEND='sparkpost.django.email_backend.SparkPostEmailBackend',
    SPARKPOST_API_KEY=API_KEY
)


def get_params(overrides=None):
    if overrides is None:
        overrides = {}

    defaults = {
        'subject': 'test subject',
        'message': 'test body',
        'from_email': 'from@example.com',
        'recipient_list': ['to@example.com'],
    }

    params = defaults.copy()
    params.update(overrides)
    return params


def mailer(params):
    return send_mail(**params)


def test_password_retrieval():
    backend = SparkPostEmailBackend()
    assert backend.client.api_key == API_KEY


def test_fail_silently():
    # should not raise
    with mock.patch.object(Transmissions, 'send') as mock_send:
        mock_send.side_effect = Exception('i should not be raised')
        mailer(get_params({'fail_silently': True}))

    # should raise
    with mock.patch.object(Transmissions, 'send') as mock_send:
        mock_send.side_effect = Exception('i should be raised')
        with pytest.raises(Exception):
            mailer(get_params())


def test_successful_sending():
    with mock.patch.object(Transmissions, 'send') as mock_send:
        mock_send.return_value = {'total_accepted_recipients': 1,
                                  'total_rejected_recipients': 2}

        result = mailer(get_params({
            'recipient_list': ['to1@example.com', 'to2@example.com'],
            'fail_silently': True
        }))
        assert result == 1

    with mock.patch.object(Transmissions, 'send') as mock_send:
        mock_send.return_value = {'total_accepted_recipients': 10,
                                  'total_rejected_recipients': 2}

        result = mailer(
            get_params(
                {'recipient_list': ['to1@example.com', 'to2@example.com'],
                 'fail_silently': True
                 }
            ))

        assert result == 10


def test_send_number_of_emails_correctly():
    with mock.patch.object(Transmissions, 'send') as mock_send:
        mailer(get_params({
            'recipient_list': ['to1@example.com', 'to2@example.com'],
            'fail_silently': True
        }))
        assert mock_send.call_count == 1

    with mock.patch.object(Transmissions, 'send') as mock_send:
        message1 = ('message 1 subject', 'message 1 body', 'from@example.com',
                    ['to1@example.com'])
        message2 = ('message 2 subject', 'message 2 body', 'from@example.com',
                    ['to2@example.com'])
        message3 = ('message 3 subject', 'message 3 body', 'from@example.com',
                    ['to3@example.com'])

        send_mass_mail((message1, message2, message3), fail_silently=False)
        assert mock_send.call_count == 3


def test_params():
    recipients = ['to1@example.com', 'to2@example.com']
    with mock.patch.object(Transmissions, 'send'):
        mailer(get_params(
            {'recipient_list': recipients,
             'fail_silently': True
             }
        ))

        Transmissions.send.assert_called_with(recipients=recipients,
                                              text='test body',
                                              from_email='from@example.com',
                                              subject='test subject'
                                              )


def test_content_types():
    def new_send(**kwargs):
        assert kwargs['text'] == 'hello there'
        assert kwargs['html'] == '<p>Hello There</p>'

        return {
            'total_accepted_recipients': 0,
            'total_rejected_recipients': 0
        }

    with mock.patch.object(Transmissions, 'send') as mock_send:
        mock_send.side_effect = new_send
        send_mail(
            'test subject',
            'hello there',
            'from@example.com',
            ['to@example.com'],
            html_message='<p>Hello There</p>'
        )


def test_send_plain_mail_after_html_mail():
    SPARKPOST_OPTIONS = {
        'track_opens': False,
        'track_clicks': False,
        'transactional': True,
    }

    reconfigure_settings(SPARKPOST_OPTIONS=SPARKPOST_OPTIONS)

    def new_send(**kwargs):
        assert kwargs['text'] == 'hello there'
        assert kwargs['html'] == '<p>Hello There</p>'

        return {
            'total_accepted_recipients': 0,
            'total_rejected_recipients': 0
        }

    def new_send_text_only(**kwargs):
        assert kwargs['text'] == 'hello there again in text only'
        assert "html" not in kwargs

        return {
            'total_accepted_recipients': 0,
            'total_rejected_recipients': 0
        }

    with mock.patch.object(Transmissions, 'send') as mock_send:
        mock_send.side_effect = new_send
        send_mail(
            'test subject',
            'hello there',
            'from@example.com',
            ['to@example.com'],
            html_message='<p>Hello There</p>'
        )
        mock_send.side_effect = new_send_text_only
        send_mail(
            'test subject 2',
            'hello there again in text only',
            'from@example.com',
            ['to@example.com'],
        )


def test_unsupported_content_types():
    params = get_params()

    with pytest.raises(UnsupportedContent):
        mail = EmailMultiAlternatives(
            params['subject'],
            'plain text',
            params['from_email'],
            params['recipient_list'])
        mail.attach_alternative('<ppp>non-plain content</ppp>', 'text/alien')
        mail.send()


def test_settings_options():
    SPARKPOST_OPTIONS = {
        'track_opens': False,
        'track_clicks': False,
        'transactional': True,
    }

    reconfigure_settings(SPARKPOST_OPTIONS=SPARKPOST_OPTIONS)

    with mock.patch.object(Transmissions, 'send'):
        mailer(get_params())
        expected_kargs = get_params().copy()
        expected_kargs["text"] = expected_kargs["message"]
        expected_kargs["recipients"] = expected_kargs["recipient_list"]
        del expected_kargs["message"]
        del expected_kargs["recipient_list"]
        expected_kargs.update(SPARKPOST_OPTIONS)
        Transmissions.send.assert_called_with(**expected_kargs)



import json

from .base import Resource


class SuppressionList(Resource):
    """
    SuppressionList class used to search, get and modify suppression status.
    For detailed request and response formats, see the `Suppresion List API
    documentation
    <https://www.sparkpost.com/api#/reference/suppression-list>`_.
    """

    key = 'suppression-list'

    def list(self, **kwargs):
        """
        List supression list entries based on the supplied parameters

        :param datetime from_date: DateTime to start listing
        :param datetime to_date: DateTime to end listing
        :param list types: Types of entries to return
        :param int limit: Maximum number of entries to return

        :returns: a ``list`` of entries
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        key_map = {
            'from_date': 'from',
            'to_date': 'to',
            'types': 'types',
            'sources': 'sources',
            'limit': 'limit'
        }
        params = dict([(key_map[i], kwargs[i]) for i in list(key_map.keys())
                       if i in kwargs])
        results = self.request('GET', self.uri, params=params)
        return results

    def get(self, email):
        """
        Retrieve a suppression list entry for a specific recipient by email

        :param str email: Email of the recipient whose status you want to
            check_status

        :returns: a suppression list entry
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        uri = "%s/%s" % (self.uri, email)
        results = self.request('GET', uri)
        return results

    def _upsert(self, status):
        uri = self.uri
        if isinstance(status, dict):
            # single upsert, update uri and remove email property
            uri = "%s/%s" % (self.uri, status.pop("email", None))
        else:
            status = {"recipients": status}
        results = self.request('PUT', uri, data=json.dumps(status))
        return results

    def create(self, entry):
        """
        Create a suppression list entry.

        :param dict|list status: If dict it is a single entry to create
            ``{
            'email': 'test@test.com',
            'transactional': True,
            'non_transactional': True,
            'description': 'Test description'
            }``, if list it is multiple entries to create

        :returns: a ``dict`` with a message
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        return self._upsert(entry)

    def update(self, entry):
        """
        Update a suppression list entry.

        :param dict|list status: If dict it is a single entry to update
            ``{
            'email': 'test@test.com',
            'transactional': True,
            'non_transactional': True,
            'description': 'Test description'
            }``, if list it is multiple entries to update

        :returns: a ``dict`` with a message
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        return self._upsert(entry)

    def delete(self, email):
        """
        Delete the suppression status for a specific recipient by email

        :param str email: Email of the recipient whose status you want to
            remove

        :returns: TODO
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        uri = "%s/%s" % (self.uri, email)
        results = self.request('DELETE', uri)
        return results



import sparkpost

from .exceptions import SparkPostAPIException


class RequestsTransport(object):
    def __init__(self):
        import requests
        self.sess = requests.Session()

    def request(self, method, uri, headers, **kwargs):
        response = self.sess.request(method, uri, headers=headers, **kwargs)
        if response.status_code == 204:
            return True
        if not response.ok:
            raise SparkPostAPIException(response)
        if 'results' in response.json():
            return response.json()['results']
        return response.json()


class Resource(object):
    key = ""

    def __init__(self, base_uri, api_key, transport_class=RequestsTransport):
        self.base_uri = base_uri
        self.api_key = api_key
        self.transport = transport_class()

    @property
    def uri(self):
        return "%s/%s" % (self.base_uri, self.key)

    def request(self, method, uri, **kwargs):
        headers = {
            'User-Agent': 'python-sparkpost/' + sparkpost.__version__,
            'Content-Type': 'application/json',
            'Authorization': self.api_key
        }
        response = self.transport.request(method, uri, headers=headers,
                                          **kwargs)
        return response

    def get(self):
        raise NotImplementedError

    def list(self):
        raise NotImplementedError

    def create(self):
        raise NotImplementedError

    def update(self):
        raise NotImplementedError

    def delete(self):
        raise NotImplementedError



import mimetypes
from base64 import b64encode

from django.core.mail import EmailMultiAlternatives
from django.core.mail.message import DEFAULT_ATTACHMENT_MIME_TYPE
from django.conf import settings

from .exceptions import UnsupportedContent


class SparkPostMessage(dict):
    """
    Takes a Django EmailMessage and formats it for use with the SparkPost API.

    The dictionary returned would be formatted like this:

    {
        'recipients': ['recipient@example.com'],
        'from_email': 'from@example.com',
        'text': 'Hello world',
        'html': '<p>Hello world</p>',
        'subject': 'Hello from the SparkPost Django email backend'
    }
    """

    def __init__(self, message):
        formatted = dict()

        if message.to:
            formatted['recipients'] = message.to

        if message.from_email:
            formatted['from_email'] = message.from_email

        if message.subject:
            formatted['subject'] = message.subject

        if hasattr(message, 'template'):
            formatted['template'] = message.template
        elif message.content_subtype == 'html':
            formatted['html'] = message.body
        else:
            formatted['text'] = message.body

        if message.cc:
            formatted['cc'] = message.cc

        if message.bcc:
            formatted['bcc'] = message.bcc

        if hasattr(message, 'reply_to') and message.reply_to:
            formatted['reply_to'] = ','.join(message.reply_to)

        if isinstance(message, EmailMultiAlternatives):
            for alternative in message.alternatives:
                if alternative[1] == 'text/html':
                    formatted['html'] = alternative[0]
                else:
                    raise UnsupportedContent(
                        'Content type %s is not supported' % alternative[1]
                    )

        if message.attachments:
            formatted['attachments'] = []
            str_encoding = settings.DEFAULT_CHARSET
            for attachment in message.attachments:
                filename, content, mimetype = attachment

                if mimetype is None:
                    mimetype, _ = mimetypes.guess_type(filename)
                    if mimetype is None:
                        mimetype = DEFAULT_ATTACHMENT_MIME_TYPE

                try:
                    if isinstance(content, unicode):
                        content = content.encode(str_encoding)
                except NameError:
                    if isinstance(content, str):
                        content = content.encode(str_encoding)
                base64_encoded_content = b64encode(content)
                formatted['attachments'].append({
                    'name': filename,
                    'data': base64_encoded_content.decode('ascii'),
                    'type': mimetype
                })

        if hasattr(message, 'substitution_data'):
            formatted['substitution_data'] = message.substitution_data

        if hasattr(message, 'campaign'):
            formatted['campaign'] = message.campaign

        if message.extra_headers:
            formatted['custom_headers'] = message.extra_headers
            if 'X-MSYS-API' in message.extra_headers:
                import json
                msys_api = json.loads(message.extra_headers['X-MSYS-API'])
                if msys_api and msys_api.get('options', {}).get('transactional', False):  # noqa: E501
                    formatted['transactional'] = True

        super(SparkPostMessage, self).__init__(formatted)



from sparkpost import SparkPost

sp = SparkPost()

response = sp.templates.create(
    id='TEST_ID',
    name='Test Template',
    from_email='test@test.com',
    subject='Test email template!',
    html='<b>This is a test email template!</b>'
)

print(response)



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.mail.message import EmailMessage
from django.utils.functional import empty

from sparkpost.django.message import SparkPostMessage
from .utils import at_least_version


def reconfigure_settings(**new_settings):
    old_settings = settings._wrapped

    settings._wrapped = empty
    settings.configure(default_settings=old_settings, **new_settings)


reconfigure_settings(
    DEFAULT_CHARSET='utf-8'
)


base_options = dict(
    subject='Test',
    body='Testing',
    from_email='test@from.com',
    to=['recipient@example.com']
)


def message(**options):
    options.update(base_options)
    email_message = EmailMessage(**options)
    return SparkPostMessage(email_message)


def multipart_message(**options):
    options.update(base_options)
    email_message = EmailMultiAlternatives(**options)
    email_message.attach_alternative('<p>Testing</p>', 'text/html')
    return SparkPostMessage(email_message)


base_expected = dict(
    recipients=['recipient@example.com'],
    from_email='test@from.com',
    subject='Test',
    text='Testing'
)


def test_minimal():
    assert message() == base_expected


def test_multipart():
    expected = dict(
        html='<p>Testing</p>'
    )
    expected.update(base_expected)
    assert multipart_message() == expected


def test_cc_bcc():
    expected = dict(
        cc=['ccone@example.com'],
        bcc=['bccone@example.com']
    )
    expected.update(base_expected)

    options = dict(cc=['ccone@example.com'],
                   bcc=['bccone@example.com'])
    assert message(**options) == expected


def test_attachment():
    email_message = EmailMessage(**base_options)
    email_message.attach('file.txt', 'test content', 'text/plain')

    actual = SparkPostMessage(email_message)
    expected = dict(
        attachments=[
            {
                'name': 'file.txt',
                'data': 'dGVzdCBjb250ZW50',
                'type': 'text/plain'
            }
        ]
    )
    expected.update(base_expected)
    assert actual == expected


def test_attachment_unicode():
    email_message = EmailMessage(**base_options)
    email_message.attach('file.txt', u'test content', 'text/plain')

    actual = SparkPostMessage(email_message)
    expected = dict(
        attachments=[
            {
                'name': 'file.txt',
                'data': 'dGVzdCBjb250ZW50',
                'type': 'text/plain'
            }
        ]
    )
    expected.update(base_expected)
    assert actual == expected


def test_attachment_guess_mimetype():
    email_message = EmailMessage(**base_options)
    email_message.attach('file.txt', 'test content')

    actual = SparkPostMessage(email_message)
    expected = dict(
        attachments=[
            {
                'name': 'file.txt',
                'data': 'dGVzdCBjb250ZW50',
                'type': 'text/plain'
            }
        ]
    )
    expected.update(base_expected)
    assert actual == expected


def test_attachment_guess_mimetype_fallback():
    email_message = EmailMessage(**base_options)
    email_message.attach('file', 'test content')

    actual = SparkPostMessage(email_message)
    expected = dict(
        attachments=[
            {
                'name': 'file',
                'data': 'dGVzdCBjb250ZW50',
                'type': 'application/octet-stream'
            }
        ]
    )
    expected.update(base_expected)
    assert actual == expected


def test_content_subtype():
    email_message = EmailMessage(
        to=['to@example.com'],
        from_email='test@from.com',
        body='<p>Testing</p>'
    )
    email_message.content_subtype = 'html'
    actual = SparkPostMessage(email_message)
    expected = dict(
        recipients=['to@example.com'],
        from_email='test@from.com',
        html='<p>Testing</p>'
    )
    assert actual == expected


def test_template():
    email_message = EmailMessage(
        to=['to@example.com'],
        from_email='test@from.com'
    )
    email_message.template = 'template-id'
    actual = SparkPostMessage(email_message)
    expected = dict(
        recipients=['to@example.com'],
        from_email='test@from.com',
        template='template-id'
    )
    assert actual == expected


def test_campaign():
    email_message = EmailMessage(**base_options)
    email_message.campaign = 'campaign-id'
    actual = SparkPostMessage(email_message)
    expected = dict(
        campaign='campaign-id'
    )
    expected.update(base_expected)
    assert actual == expected


def test_substitution_data():
    email_message = EmailMessage(
        to=[
            {
                "address": "to@example.com",
                "substitution_data": {
                    "key": "value"
                }
            }
        ],
        from_email='test@from.com'
    )
    email_message.template = 'template-id'
    email_message.substitution_data = {"key2": "value2"}
    actual = SparkPostMessage(email_message)

    expected = dict(
        recipients=[
            {
                "address": "to@example.com",
                "substitution_data": {
                    "key": "value"
                }
            }
        ],
        from_email='test@from.com',
        template='template-id',
        substitution_data={"key2": "value2"}
    )

    assert actual == expected


if at_least_version('1.8'):
    def test_reply_to():
        expected = dict(
            reply_to='replyone@example.com,replytwo@example.com'
        )
        expected.update(base_expected)

        assert message(reply_to=['replyone@example.com',
                                 'replytwo@example.com']) == expected


def test_extra_headers():
    email_message = EmailMessage(**base_options)
    email_message.extra_headers['FOO'] = 'bar'

    actual = SparkPostMessage(email_message)
    expected = dict(
        custom_headers={'FOO': 'bar'},
    )
    expected.update(base_expected)
    assert actual == expected


def test_transactional():
    email_message = EmailMessage(**base_options)
    import json
    msys_api = json.dumps({'options': {'transactional': True}})
    email_message.extra_headers['X-MSYS-API'] = msys_api

    actual = SparkPostMessage(email_message)
    expected = dict(
        custom_headers={'X-MSYS-API': msys_api},
        transactional=True,
    )
    expected.update(base_expected)
    assert actual == expected



import os

from sparkpost import SparkPost

parent_dir = os.path.dirname(os.path.realpath(__file__))
attachment_path = os.path.abspath(os.path.join(parent_dir, "a-file.txt"))

sp = SparkPost()

response = sp.transmissions.send(
    recipients=[
        'postmaster@example.com',
        'you@me.com',
        {
            'address': {
                'email': 'john.doe@example.com',
                'name': 'John Doe'
            }
        }
    ],
    cc=['carboncopy@example.com'],
    bcc=['blindcarboncopy@example.com'],
    html='<p>Hello {{name}}</p>',
    text='Hello {{name}}',
    from_email='Test User <test@sparkpostbox.com>',
    subject='Example Script',
    description='contrived example',
    custom_headers={
        'X-CUSTOM-HEADER': 'foo bar'
    },
    track_opens=True,
    track_clicks=True,
    attachments=[
        {
            "name": "test.txt",
            "type": "text/plain",
            "filename": attachment_path
        }
    ],
    campaign='sdk example',
    metadata={
        'key': 'value',
        'arbitrary': 'values'
    },
    substitution_data={
        'name': 'Example User'
    },
    reply_to='no-reply@sparkpostmail.com'
)



import base64
import json
import os
import tempfile

import pytest
import six

from sparkpost.tornado import SparkPost, SparkPostAPIException
from tornado import ioloop
from .utils import AsyncClientMock

responses = AsyncClientMock()


@responses.activate
def test_success_send():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    sp = SparkPost('fake-key')
    results = ioloop.IOLoop().run_sync(sp.transmission.send)
    assert results == 'yay'


@responses.activate
def test_success_send_with_attachments():
    try:
        # Let's compare unicode for Python 2 / 3 compatibility
        test_content = six.u("Hello \nWorld\n")
        (_, temp_file_path) = tempfile.mkstemp()
        with open(temp_file_path, "w") as temp_file:
            temp_file.write(test_content)

        responses.add(
            responses.POST,
            'https://api.sparkpost.com/api/v1/transmissions',
            status=200,
            content_type='application/json',
            body='{"results": "yay"}'
        )
        sp = SparkPost('fake-key')

        attachment = {
            "name": "test.txt",
            "type": "text/plain",
            "filename": temp_file_path
        }

        def send():
            return sp.transmission.send(attachments=[attachment])
        results = ioloop.IOLoop().run_sync(send)

        request_params = json.loads(responses.calls[0].request.body)
        content = base64.b64decode(
            request_params["content"]["attachments"][0]["data"])
        # Let's compare unicode for Python 2 / 3 compatibility
        assert test_content == content.decode("ascii")

        assert results == 'yay'

        attachment = {
            "name": "test.txt",
            "type": "text/plain",
            "data": base64.b64encode(
                test_content.encode("ascii")).decode("ascii")
        }

        def send():
            return sp.transmission.send(attachments=[attachment])
        results = ioloop.IOLoop().run_sync(send)

        request_params = json.loads(responses.calls[1].request.body)
        content = base64.b64decode(
            request_params["content"]["attachments"][0]["data"])
        # Let's compare unicode for Python 2 / 3 compatibility
        assert test_content == content.decode("ascii")

        assert results == 'yay'
    finally:
        os.unlink(temp_file_path)


@responses.activate
def test_fail_send():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=500,
        content_type='application/json',
        body="""
        {"errors": [{"message": "You failed", "description": "More Info"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        ioloop.IOLoop().run_sync(sp.transmission.send)


@responses.activate
def test_success_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions/foobar',
        status=200,
        content_type='application/json',
        body='{"results": {"transmission": {}}}'
    )
    sp = SparkPost('fake-key')

    def send():
        return sp.transmission.get('foobar')
    results = ioloop.IOLoop().run_sync(send, timeout=3)
    assert results == {}


@responses.activate
def test_fail_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions/foobar',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "cant find", "description": "where you go"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')

        def send():
            return sp.transmission.get('foobar')
        ioloop.IOLoop().run_sync(send, timeout=3)


@responses.activate
def test_nocontent_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=204,
        content_type='application/json',
        body=''
    )
    sp = SparkPost('fake-key')
    response = ioloop.IOLoop().run_sync(sp.transmission.list)
    assert response is True


@responses.activate
def test_brokenjson_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=200,
        content_type='application/json',
        body='{"results":'
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        ioloop.IOLoop().run_sync(sp.transmission.list)


@responses.activate
def test_noresults_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=200,
        content_type='application/json',
        body='{"ok": false}'
    )
    sp = SparkPost('fake-key')
    response = ioloop.IOLoop().run_sync(sp.transmission.list)
    assert response == {"ok": False}


@responses.activate
def test_success_list():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=200,
        content_type='application/json',
        body='{"results": []}'
    )
    sp = SparkPost('fake-key')
    response = ioloop.IOLoop().run_sync(sp.transmission.list)
    assert response == []



import sparkpost

from .exceptions import SparkPostAPIException
from .base import TornadoTransport
from .transmissions import Transmissions

__all__ = ["SparkPost", "TornadoTransport", "SparkPostAPIException",
           "Transmissions"]


class SparkPost(sparkpost.SparkPost):
    TRANSPORT_CLASS = TornadoTransport

    def __init__(self, *args, **kwargs):
        super(SparkPost, self).__init__(*args, **kwargs)
        self.transmissions = Transmissions(self.base_uri, self.api_key,
                                           self.TRANSPORT_CLASS)
        self.transmission = self.transmissions



from sparkpost import SparkPost

sp = SparkPost()

response = sp.transmissions.send(
    recipients=[
        'postmaster@example.com',
        'you@me.com',
        {
            'address': {
                'email': 'john.doe@example.com',
                'name': 'John Doe'
            }
        }
    ],
    html='<p>Hello world {{name}}</p>',
    text='Hello world {{name}}',
    from_email='test@sparkpostbox.com',
    subject='Example Script',
    description='contrived example',
    custom_headers={
        'X-CUSTOM-HEADER': 'foo bar'
    },
    track_opens=True,
    track_clicks=True,
    start_time='2015-11-06T09:10:00-05:00',
    campaign='python-sparkpost example',
    metadata={
        'key': 'value',
        'arbitrary': 'values'
    },
    substitution_data={
        'name': 'Example User'
    }
)

print(response)



import json
from tornado import gen
from tornado.httpclient import AsyncHTTPClient, HTTPError

from .exceptions import SparkPostAPIException


class TornadoTransport(object):
    @gen.coroutine
    def request(self, method, uri, headers, **kwargs):
        if "data" in kwargs:
            kwargs["body"] = kwargs.pop("data")
        client = AsyncHTTPClient()
        try:
            response = yield client.fetch(uri, method=method, headers=headers,
                                          **kwargs)
        except HTTPError as ex:
            raise SparkPostAPIException(ex.response)
        if response.code == 204:
            raise gen.Return(True)
        if response.code == 200:
            result = None
            # noinspection PyBroadException
            try:
                result = json.loads(response.body.decode("utf-8"))
            # TODO: select exception to catch here
            except:  # noqa: E722
                pass
            if result:
                if 'results' in result:
                    raise gen.Return(result['results'])
                raise gen.Return(result)
        raise SparkPostAPIException(response)



from sparkpost import SparkPost

sp = SparkPost()
response = sp.templates.update(
    'TEST_ID',
    name='Test Template',
    from_email='test@test.com',
    subject='Updated Test email template!',
    html='<b>This is a test email template! Updated!</b>'
)
print(response)



from .base import Resource, RequestsTransport


class Metrics(object):
    "Wrapper for sub-resources"

    def __init__(self, base_uri, api_key, transport_class=RequestsTransport):
        self.base_uri = "%s/%s" % (base_uri, 'metrics')
        self.campaigns = Campaigns(self.base_uri, api_key, transport_class)
        self.domains = Domains(self.base_uri, api_key, transport_class)


class Campaigns(Resource):
    key = 'campaigns'

    def list(self, **kwargs):
        results = self.request('GET', self.uri, **kwargs)
        return results['campaigns']


class Domains(Resource):
    key = 'domains'

    def list(self, **kwargs):
        results = self.request('GET', self.uri, **kwargs)
        return results['domains']



import os

from .base import RequestsTransport
from .exceptions import SparkPostException
from .metrics import Metrics
from .recipient_lists import RecipientLists
from .suppression_list import SuppressionList
from .templates import Templates
from .transmissions import Transmissions


__version__ = '1.3.6'


class SparkPost(object):
    TRANSPORT_CLASS = RequestsTransport

    def __init__(self, api_key=None, base_uri='https://api.sparkpost.com',
                 version='1'):
        "Set up the SparkPost API client"
        if not api_key:
            api_key = self.get_api_key()
            if not api_key:
                raise SparkPostException("No API key. Improve message.")

        self.base_uri = base_uri + '/api/v' + version
        self.api_key = api_key

        self.metrics = Metrics(self.base_uri, self.api_key,
                               self.TRANSPORT_CLASS)
        self.recipient_lists = RecipientLists(self.base_uri, self.api_key,
                                              self.TRANSPORT_CLASS)
        self.suppression_list = SuppressionList(self.base_uri, self.api_key,
                                                self.TRANSPORT_CLASS)
        self.templates = Templates(self.base_uri, self.api_key,
                                   self.TRANSPORT_CLASS)
        self.transmissions = Transmissions(self.base_uri, self.api_key,
                                           self.TRANSPORT_CLASS)
        # Keeping self.transmission for backwards compatibility.
        # Will be removed in a future release.
        self.transmission = self.transmissions

    def get_api_key(self):
        "Get API key from environment variable"
        return os.environ.get('SPARKPOST_API_KEY', None)



import base64
import json
import os
import tempfile
import warnings

import pytest
import responses
import six
from mock import patch

from sparkpost import SparkPost
from sparkpost import Transmissions
from sparkpost.exceptions import SparkPostAPIException, SparkPostException


def test_translate_keys_with_list():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(recipient_list='test')
    assert results['content']['use_draft_template'] is False
    assert isinstance(results['content']['headers'], dict)
    assert results['recipients'] == {'list_id': 'test'}


def test_translate_keys_with_recips():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(recipients=['test',
                                            {'key': 'value'}, 'foobar'])
    assert results['recipients'] == [{'address': {'email': 'test'}},
                                     {'key': 'value'},
                                     {'address': {'email': 'foobar'}}]


def test_exceptions_for_recipients():
    t = Transmissions('uri', 'key')
    with pytest.raises(SparkPostException):
        t._translate_keys(recipients='test')


def test_translate_keys_with_unicode_recips():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(recipients=[u'unicode_email@example.com',
                                            'str_email@example.com'])
    assert results['recipients'] == [
        {'address': {'email': 'unicode_email@example.com'}},
        {'address': {'email': 'str_email@example.com'}}
    ]


def test_translate_keys_for_email_parsing():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(recipients=['hansel@example.com',
                                            'Gretel <gretel@example.com>'])
    assert results['recipients'] == [
        {'address': {'email': 'hansel@example.com'}},
        {'address': {'name': 'Gretel', 'email': 'gretel@example.com'}}
    ]


def test_translate_keys_for_from_email():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(from_email='Testing <testing@example.com>')
    assert results['content']['from'] == {
        'name': 'Testing',
        'email': 'testing@example.com'
    }


def test_format_header_to():
    t = Transmissions('uri', 'key')
    formatted = t._format_header_to(recipient={
        'address': {'email': 'primary@example.com'}
    })
    assert formatted == 'primary@example.com'

    formatted = t._format_header_to(recipient={
        'address': {'name': 'Testing', 'email': 'primary@example.com'}
    })
    assert formatted == '"Testing" <primary@example.com>'


def test_cc_with_sub_data():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(
        recipients=[{
            'address': {'email': 'primary@example.com'},
            'substitution_data': {'fake': 'data'}
        }],
        cc=['ccone@example.com']
    )
    assert results['recipients'] == [
        {
            'address': {'email': 'primary@example.com'},
            'substitution_data': {'fake': 'data'}
        },
        {
            'address': {
                'email': 'ccone@example.com',
                'header_to': 'primary@example.com'
            },
            'substitution_data': {'fake': 'data'}
        }
    ]


def test_translate_keys_with_cc():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(recipients=['primary@example.com'],
                                cc=['ccone@example.com'])
    assert results['recipients'] == [
        {'address': {'email': 'primary@example.com'}},
        {'address': {'email': 'ccone@example.com',
                     'header_to': 'primary@example.com'}},
    ]
    assert results['content']['headers'] == {
        'CC': 'ccone@example.com'
    }


def test_translate_keys_with_multiple_cc():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(recipients=['primary@example.com'],
                                cc=['ccone@example.com', 'cctwo@example.com'])
    assert results['recipients'] == [
        {'address': {'email': 'primary@example.com'}},
        {'address': {'email': 'ccone@example.com',
                     'header_to': 'primary@example.com'}},
        {'address': {'email': 'cctwo@example.com',
                     'header_to': 'primary@example.com'}},
    ]
    assert results['content']['headers'] == {
        'CC': 'ccone@example.com,cctwo@example.com'
    }


def test_translate_keys_with_bcc():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(recipients=['primary@example.com'],
                                bcc=['bccone@example.com'])
    assert results['recipients'] == [
        {'address': {'email': 'primary@example.com'}},
        {'address': {'email': 'bccone@example.com',
                     'header_to': 'primary@example.com'}},
    ]


def test_translate_keys_with_inline_css():
    t = Transmissions('uri', 'key')
    results = t._translate_keys(inline_css=True)
    assert results['options'].get('inline_css') is True


def test_translate_keys_with_email_rfc822():
    t = Transmissions('uri', 'key')

    # Build data using implicit cat, as max line length is enforced
    eml = (
        'To: wilma <wilma@flintstone.com>\n',
        'From: fred <fred@flintstone.com>\n',
        'Subject: Bedrock declaration\n',
        'MIME-Version: 1.0\n',
        'Content-Type: text/plain; charset=utf-8; format=flowed\n',
        'Content-Transfer-Encoding: 7bit\nContent-Language: en-GB\n',
        '\n',
        'When in the Course of human events we yell yabba dabba doo.',
    )

    # Just a selection. Don't test attribs with irregular naming
    non_rfc822_attrs = {
        'headers': 'foo',
        'reply_to': 'foo',
        'subject': 'foo',
        'html': 'foo',
        'text': 'foo',
    }

    # Demonstrate that email_rfc822 overrides other content attributes
    test_content = {'email_rfc822': eml}
    test_content.update(non_rfc822_attrs)
    results = t._translate_keys(**test_content)
    for i in non_rfc822_attrs:
        assert results['content'].get(i) is None
    assert results['content'].get('email_rfc822') is not None


@responses.activate
def test_success_send():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    sp = SparkPost('fake-key')
    results = sp.transmission.send()
    assert results == 'yay'


@responses.activate
def test_success_send_with_attachments():
    try:
        # Let's compare unicode for Python 2 / 3 compatibility
        test_content = six.u("Hello \nWorld\n")
        (_, temp_file_path) = tempfile.mkstemp()
        with open(temp_file_path, "w") as temp_file:
            temp_file.write(test_content)

        responses.add(
            responses.POST,
            'https://api.sparkpost.com/api/v1/transmissions',
            status=200,
            content_type='application/json',
            body='{"results": "yay"}'
        )
        sp = SparkPost('fake-key')

        attachment = {
            "name": "test.txt",
            "type": "text/plain",
            "filename": temp_file_path
        }
        results = sp.transmission.send(attachments=[attachment])

        request_params = json.loads(responses.calls[0].request.body)
        content = base64.b64decode(
            request_params["content"]["attachments"][0]["data"])
        # Let's compare unicode for Python 2 / 3 compatibility
        assert test_content == content.decode("ascii")

        assert results == 'yay'

        attachment = {
            "name": "test.txt",
            "type": "text/plain",
            "data": base64.b64encode(
                test_content.encode("ascii")).decode("ascii")
        }
        results = sp.transmission.send(attachments=[attachment])

        request_params = json.loads(responses.calls[1].request.body)
        content = base64.b64decode(
            request_params["content"]["attachments"][0]["data"])
        # Let's compare unicode for Python 2 / 3 compatibility
        assert test_content == content.decode("ascii")

        assert results == 'yay'
    finally:
        os.unlink(temp_file_path)


@responses.activate
def test_success_send_with_inline_images():
    current_dir = os.path.abspath(os.path.dirname(__file__))
    image_path = os.path.join(current_dir, 'assets', 'sparkpostdev.png')

    with open(image_path, "rb") as image:
        encoded_image = base64.b64encode(image.read()).decode("ascii")

    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=200,
        content_type='application/json',
        body='{"results": "yay"}'
    )
    sp = SparkPost('fake-key')

    image_data = {
        "name": "sparkpostdev",
        "type": "image/png",
        "filename": image_path
    }
    results = sp.transmission.send(inline_images=[image_data])
    request_params = json.loads(responses.calls[0].request.body)
    content = request_params["content"]["inline_images"][0]["data"]

    assert encoded_image == content
    assert results == 'yay'

    image_data = {
        "name": "sparkpostdev",
        "type": "image/png",
        "data": encoded_image
    }
    results = sp.transmission.send(inline_images=[image_data])
    request_params = json.loads(responses.calls[1].request.body)
    content = request_params["content"]["inline_images"][0]["data"]

    assert content == encoded_image
    assert results == 'yay'


@responses.activate
def test_fail_send():
    responses.add(
        responses.POST,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=500,
        content_type='application/json',
        body="""
        {"errors": [{"message": "You failed", "description": "More Info"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.transmission.send()


@responses.activate
def test_success_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions/foobar',
        status=200,
        content_type='application/json',
        body='{"results": {"transmission": {}}}'
    )
    sp = SparkPost('fake-key')
    results = sp.transmission.get('foobar')
    assert results == {}


@responses.activate
def test_fail_get():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions/foobar',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "cant find", "description": "where you go"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.transmission.get('foobar')


@responses.activate
@patch.object(warnings, 'warn')
def test_success_list(mock_warn):
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions',
        status=200,
        content_type='application/json',
        body='{"results": []}'
    )
    sp = SparkPost('fake-key')
    response = sp.transmission.list()
    assert mock_warn.called
    assert response == []


@responses.activate
def test_success_list_with_params():
    responses.add(
        responses.GET,
        'https://api.sparkpost.com/api/v1/transmissions?template_id=abcd',
        status=200,
        content_type='application/json',
        body='{"results": []}',
        match_querystring=True

    )
    sp = SparkPost('fake-key')
    response = sp.transmission.list(template_id='abcd')
    assert response == []


@responses.activate
def test_success_delete():
    responses.add(
        responses.DELETE,
        'https://api.sparkpost.com/api/v1/transmissions/foobar',
        status=200,
        content_type='application/json',
        body='{}'
    )
    sp = SparkPost('fake-key')
    results = sp.transmission.delete('foobar')
    assert results == {}


@responses.activate
def test_fail_delete():
    responses.add(
        responses.DELETE,
        'https://api.sparkpost.com/api/v1/transmissions/foobar',
        status=404,
        content_type='application/json',
        body="""
        {"errors": [{"message": "resource not found",
            "description": "Resource not found:transmission id foobar"}]}
        """
    )
    with pytest.raises(SparkPostAPIException):
        sp = SparkPost('fake-key')
        sp.transmission.delete('foobar')



from __future__ import print_function
from collections import namedtuple

from responses import RequestsMock
from tornado import ioloop
from tornado.concurrent import Future
from tornado.httpclient import HTTPRequest, HTTPResponse, HTTPError

Request = namedtuple("Request", ["url", "method", "headers", "body"])


class ResponseGenerator(object):
    def get_adapter(self, url):
        return self

    def build_response(self, request, response):
        resp = HTTPResponse(request, response.status, headers=response.headers,
                            effective_url=request.url, error=None, buffer="")
        resp._body = response.data
        f = Future()
        f.content = None
        if response.status < 200 or response.status >= 300:
            resp.error = HTTPError(response.status, response=resp)
            ioloop.IOLoop().current().add_callback(f.set_exception, resp.error)
        else:
            ioloop.IOLoop().current().add_callback(f.set_result, resp)
        return f


class AsyncClientMock(RequestsMock):
    def start(self):
        import mock

        def unbound_on_send(client, request, callback=None, **kwargs):
            if not isinstance(request, HTTPRequest):
                request = Request(request,
                                  kwargs.get("method", "GET"),
                                  kwargs.get("headers", []),
                                  kwargs.get("body", ""))
            return self._on_request(ResponseGenerator(), request)
        self._patcher = mock.patch('tornado.httpclient.AsyncHTTPClient.fetch',
                                   unbound_on_send)
        self._patcher.start()

    def stop(self):
        self._patcher.stop()



import pytest
import responses

from sparkpost.base import Resource
from sparkpost.exceptions import SparkPostAPIException


fake_base_uri = 'https://fake-base.com'
fake_api_key = 'fake-api-key'
fake_resource_key = 'fake-resource-key'
fake_uri = "%s/%s" % (fake_base_uri, fake_resource_key)


def create_resource():
    resource = Resource(fake_base_uri, fake_api_key)
    resource.key = fake_resource_key
    return resource


def test_uri():
    resource = create_resource()
    assert resource.uri == fake_uri


@responses.activate
def test_success_request():
    responses.add(
        responses.GET,
        fake_uri,
        status=200,
        content_type='application/json',
        body='{}'
    )
    resource = create_resource()
    results = resource.request('GET', resource.uri)
    assert results == {}


@responses.activate
def test_success_request_with_results():
    responses.add(
        responses.GET,
        fake_uri,
        status=200,
        content_type='application/json',
        body='{"results": []}'
    )
    resource = create_resource()
    results = resource.request('GET', resource.uri)
    assert results == []


@responses.activate
def test_fail_request():
    responses.add(
        responses.GET,
        fake_uri,
        status=500,
        content_type='application/json',
        body='{"errors": [{"message": "failure", "description": "desc"}]}'
    )
    resource = create_resource()
    with pytest.raises(SparkPostAPIException):
        resource.request('GET', resource.uri)


@responses.activate
def test_fail_wrongjson_request():
    responses.add(
        responses.GET,
        fake_uri,
        status=500,
        content_type='application/json',
        body='{"errors": ["Error!"]}'
    )
    resource = create_resource()
    with pytest.raises(SparkPostAPIException):
        resource.request('GET', resource.uri)


@responses.activate
def test_fail_nojson_request():
    responses.add(
        responses.GET,
        fake_uri,
        status=500,
        content_type='application/json',
        body='{"errors": '
    )
    resource = create_resource()
    with pytest.raises(SparkPostAPIException):
        resource.request('GET', resource.uri)


@responses.activate
def test_fail_no_errors():
    responses.add(
        responses.GET,
        fake_uri,
        status=500,
        content_type='application/json',
        body='no errors'
    )
    resource = create_resource()
    with pytest.raises(SparkPostAPIException):
        resource.request('GET', resource.uri)


def test_fail_get():
    resource = create_resource()
    with pytest.raises(NotImplementedError):
        resource.get()


def test_fail_list():
    resource = create_resource()
    with pytest.raises(NotImplementedError):
        resource.list()


def test_fail_create():
    resource = create_resource()
    with pytest.raises(NotImplementedError):
        resource.create()


def test_fail_update():
    resource = create_resource()
    with pytest.raises(NotImplementedError):
        resource.update()


def test_fail_delete():
    resource = create_resource()
    with pytest.raises(NotImplementedError):
        resource.delete()



from codecs import open
from setuptools import setup, find_packages

with open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()

setup(
    name='sparkpost',
    version='1.3.7',
    author='SparkPost',
    author_email='developers@sparkpost.com',
    packages=find_packages(),
    url='https://github.com/SparkPost/python-sparkpost',
    license='Apache 2.0',
    description='SparkPost Python API client',
    long_description=readme,
    install_requires=['requests>=2.20.1'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Communications :: Email',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



from sparkpost import SparkPost

sp = SparkPost()

response = sp.transmissions.send(
    recipient_list='my_list',
    html='<p>Hello world {{name}}</p>',
    text='Hello world {{name}}',
    from_email='test@sparkpostbox.com',
    subject='Example Script',
    description='contrived example',
    custom_headers={
        'X-CUSTOM-HEADER': 'foo bar'
    },
    campaign='sdk example',
    metadata={
        'key': 'value',
        'arbitrary': 'values'
    },
    substitution_data={
        'name': 'Example User'
    },
    reply_to='no-reply@sparkpostmail.com'
)



import base64
import copy
import json
import warnings
from email.utils import parseaddr

from .base import Resource
from .exceptions import SparkPostException


try:
    string_types = basestring
except NameError:
    string_types = str  # Python 3 doesn't have basestring


class Transmissions(Resource):
    """
    Transmission class used to send, list and get transmissions. For detailed
    request and response formats, see the `Transmissions API documentation
    <https://www.sparkpost.com/docs/transmissions-api>`_.
    """

    key = 'transmissions'

    def _translate_keys(self, **kwargs):
        model = {
            'content': {},
            'options': {},
            'recipients': {}
        }

        model['description'] = kwargs.get('description')
        model['return_path'] = kwargs.get('return_path')
        model['campaign_id'] = kwargs.get('campaign')
        model['metadata'] = kwargs.get('metadata')
        model['substitution_data'] = kwargs.get('substitution_data')

        model['options']['start_time'] = kwargs.get('start_time')
        model['options']['open_tracking'] = kwargs.get('track_opens')
        model['options']['click_tracking'] = kwargs.get('track_clicks')
        model['options']['transactional'] = kwargs.get('transactional')
        model['options']['sandbox'] = kwargs.get('use_sandbox')
        model['options']['skip_suppression'] = kwargs.get('skip_suppression')
        model['options']['ip_pool'] = kwargs.get('ip_pool')
        model['options']['inline_css'] = kwargs.get('inline_css')

        rfc822 = kwargs.get('email_rfc822')
        if rfc822:
            model['content']['email_rfc822'] = rfc822
        else:
            model['content']['headers'] = kwargs.get('custom_headers', {})
            model['content']['use_draft_template'] = \
                kwargs.get('use_draft_template', False)
            model['content']['reply_to'] = kwargs.get('reply_to')
            model['content']['subject'] = kwargs.get('subject')
            from_email = kwargs.get('from_email')
            if isinstance(from_email, string_types):
                from_email = self._parse_address(from_email)
            model['content']['from'] = from_email
            model['content']['html'] = kwargs.get('html')
            model['content']['text'] = kwargs.get('text')
            model['content']['template_id'] = kwargs.get('template')

            attachments = kwargs.get('attachments', [])
            model['content']['attachments'] = self._extract_attachments(
                attachments)

            if 'inline_images' in kwargs:
                inline_images = kwargs['inline_images']
                model['content']['inline_images'] = self._extract_attachments(
                    inline_images)

        recipient_list = kwargs.get('recipient_list')
        if recipient_list:
            model['recipients']['list_id'] = recipient_list
        else:
            recipients = kwargs.get('recipients', [])
            recipients = self._extract_recipients(recipients)
            cc = kwargs.get('cc')
            bcc = kwargs.get('bcc')

            if cc:
                model['content']['headers']['CC'] = ','.join(cc)
                cc_copies = self._format_copies(recipients, cc)
                recipients = recipients + cc_copies
            if bcc:
                bcc_copies = self._format_copies(recipients, bcc)
                recipients = recipients + bcc_copies

            model['recipients'] = recipients

        return model

    def _format_copies(self, recipients, copies):
        formatted_copies = []
        if len(recipients) > 0:
            formatted_copies = self._extract_recipients(copies)
            main_recipient = copy.deepcopy(recipients[0])
            main_recipient.pop('address')
            for recipient in formatted_copies:
                recipient['address'].update({
                    'header_to': self._format_header_to(recipients[0])
                })
                recipient.update(**main_recipient)
        return formatted_copies

    def _format_header_to(self, recipient):
        if 'name' in recipient['address']:
            return '"{name}" <{email}>'.format(
                name=recipient['address']['name'],
                email=recipient['address']['email']
            )
        return recipient['address']['email']

    def _extract_attachments(self, attachments):
        formatted_attachments = []
        for attachment in attachments:
            formatted_attachment = {}
            formatted_attachment['type'] = attachment.get('type')
            formatted_attachment['name'] = attachment.get('name')
            if 'filename' in attachment:
                formatted_attachment['data'] = self._get_base64_from_file(
                    attachment['filename'])
            else:
                formatted_attachment['data'] = attachment.get('data')
            formatted_attachments.append(formatted_attachment)
        return formatted_attachments

    def _get_base64_from_file(self, filename):
        with open(filename, "rb") as a_file:
            encoded_string = base64.b64encode(a_file.read()).decode("ascii")
        return encoded_string

    def _parse_address(self, address):
        name, email = parseaddr(address)
        parsed_address = {
            'email': email
        }
        if name:
            parsed_address['name'] = name
        return parsed_address

    def _extract_recipients(self, recipients):

        if not (isinstance(recipients, (list, dict))):
            raise SparkPostException('recipients must be a list or dict')

        formatted_recipients = []
        for recip in recipients:
            if isinstance(recip, string_types):
                formatted_recipients.append({
                    'address': self._parse_address(recip)
                })
            else:
                formatted_recipients.append(recip)
        return formatted_recipients

    def send(self, **kwargs):
        """
        Send a transmission based on the supplied parameters

        :param list|dict recipients: List of email addresses, or a dict:
            ``{'address': {'name': 'Kyla', 'email': 'hello@example.com' }}``
        :param str recipient_list: ID of recipient list. If this is set,
            the `recipients` param will be ignored
        :param cc: List of email addresses to send carbon copy to
        :param bcc: List of email addresses to send blind carbon copy to
        :param str template: ID of template to be used. Setting a template
            overrides the HTML and text params
        :param bool use_draft_template: Defaults to False. Set to true if you
            want to send a template that is a draft
        :param str html: HTML part of transmission
        :param str text: Text part of transmission
        :param str subject: Subject of transmission
        :param str from_email: Email that the transmission comes from. The
            domain must be a verified sending domain belonging to your account
            or the transmission will fail. You can pass a from email or both
            from name and from email - `testing@example.com` or
            `Test Email <testing@example.com>` will both work.
        :param str reply_to: Reply to of transmission
        :param str return_path: Email address to use for envelope FROM. The
            domain part of the return_path address must be a
            CNAME-verified sending domain.
        :param str description: Description of transmission
        :param str campaign: Campaign of transmission
        :param dict metadata: Any data you want to send along with
            transmission, used in WebHooks
        :param dict substitution_data: Corresponds to substitutions in
            html/text content. See `substitutions reference
            <https://developers.sparkpost.com/api/substitutions-reference>`_.
        :param attachments: List of dicts. For example:

            .. code-block:: python

                dict(
                    type='application/pdf',
                    name='document.pdf',
                    data='base64 encoded string'
                )

            Replace `data` with `filename` if you want the library to perform
            the base64 conversion. For example:

            .. code-block:: python

                dict(
                    type='application/pdf',
                    name='document.pdf',
                    filename='/full/path/to/document.pdf'
                )
        :param inline_images: List of dicts. For example:

            .. code-block:: python

                dict(
                    type='image/png',
                    name='imageCID',
                    data='base64 encoded string'
                )

            Replace `data` with `filename` if you want the library to perform
            the base64 conversion. For example:

            .. code-block:: python

                dict(
                    type='image/png',
                    name='imageCID',
                    filename='/full/path/to/image.png'
                )

        :param str start_time: Delay generation of messages until this
            datetime. Format YYYY-MM-DDTHH:MM:SS+-HH:MM. Example:
            '2015-02-11T08:00:00-04:00'.
        :param bool track_opens: Defaults to True. Used to track opens of
            transmission
        :param bool track_clicks: Defaults to True. Used to track clicks of
            transmission
        :param bool use_sandbox: Flag must be set to use sandbox domain instead
            of verified sending domain. Limited to a lifetime of 50
            transmissions with this domain
        :param bool transactional: Whether message is transactional or
            non-transactional for unsubscribe and suppression purposes
        :param bool skip_suppression: Whether or not to ignore customer
            suppression rules, for this transmission only. Only applicable if
            your configuration supports this parameter. (Enterprise only)
        :param str ip_pool: The ID of an IP pool associated with
            your account
        :param bool inline_css: Whether or not to perform CSS inlining
        :param dict custom_headers: Used to set any headers associated with
            transmission. See `header notes
            <https://developers.sparkpost.com/api/transmissions.html#header-header-notes>`_

        :returns: a ``dict`` with the transmission ID and number of accepted
            and rejected recipients
        :raises: :exc:`SparkPostAPIException` if transmission cannot be sent
        """

        payload = self._translate_keys(**kwargs)
        data = json.dumps(payload)
        results = self.request('POST', self.uri, data=data)
        return results

    def _fetch_get(self, transmission_id):
        uri = "%s/%s" % (self.uri, transmission_id)
        results = self.request('GET', uri)
        return results

    def get(self, transmission_id):
        """
        Get a transmission by ID

        :param str transmission_id: ID of the transmission you want to retrieve

        :returns: the requested transmission, if found
        :raises: :exc:`SparkPostAPIException` if transmission is not found
        """
        results = self._fetch_get(transmission_id)
        return results['transmission']

    def list(self, **kwargs):
        """
        Get a list of your transmissions. This method is deprecated.

        :param campaign_id: ID of the campaign used by the transmissions
        :param template_id: ID of the template used by the transmissions

        :returns: list of transmissions
        :raises: :exc:`SparkPostAPIException` if API call fails
        """
        warn_msg = 'This method is deprecated.'

        warnings.warn(warn_msg, DeprecationWarning)
        return self.request('GET', self.uri, params=kwargs)

    def delete(self, transmission_id):
        """
        Delete a transmission by ID

        :param str transmission_id: ID of the transmission you want to delete

        :returns: {}  if transmission is deleted
        :raises: :exc:`SparkPostAPIException` if transmission is not found
            or Canceled
        """
        uri = "%s/%s" % (self.uri, transmission_id)
        results = self.request('DELETE', uri)
        return results



import os

from sparkpost.base import Resource


class Webhooks(Resource):
    key = "webhooks"

    def list(self, **kwargs):
        results = self.request('GET', self.uri, **kwargs)
        return results


api_key = os.environ.get('SPARKPOST_API_KEY', None)
webhooks = Webhooks('https://api.sparkpost.com/api/v1', api_key)

# returns a list of webhooks for your account
print(webhooks.list())
